# 支付服务

## PayController

### pay
> 省略

### payBack
> 退款
```java
// 删除个人详细信息缓存
if (redisUtils.hasKey(userInfoKey)) {
    redisUtils.del(userInfoKey);
}
// 删除未支付订单列表缓存
if (redisUtils.hasKey(noTakeKey)) {
    redisUtils.del(noTakeKey);
}
// 删除订单详情缓存
if (redisUtils.hasKey(selectOrderKey)) {
    redisUtils.del(selectOrderKey);
}
// 删除场次详情缓存
if (redisUtils.hasKey(countDetailKey)) {
    redisUtils.del(countDetailKey);
}
```

## PayServiceImpl

### pay
```java
public PayResponse pay(PayRequset requset) {
    PayResponse payResponse = new PayResponse();
    Long userId = requset.getUserId();
    Double userMoney = null;
    try {
        // 1. 先核对支付密码是否正确
        tag = MqTags.PAY_CHECK_CANCLE.getTag();
        String key = RedisConstants.USER_INFO_EXPIRE.getKey() + userId;
        UserResponse userResponse = new UserResponse();
        if (redisUtils.hasKey(key)) {
            userResponse = (UserResponse) redisUtils.get(key);
        } else {
            UserRequest request = new UserRequest();
            request.setId(userId);
            // 获取用户信息
            userResponse = userService.getUserById(request);
        }

        // 支付密码不对
        if (!userResponse.getUserDto().getPayPassword().equals(requset.getPayPassword())) {
            payResponse.setCode(SbCode.PAY_PASSWORD_ERROR.getCode());
            payResponse.setMsg(SbCode.PAY_PASSWORD_ERROR.getMessage());
            return payResponse;
        }
        // 2。 核对余额是否够
        userMoney = userResponse.getUserDto().getMoney();
        Double subMoney = NumberUtil.sub(userMoney, requset.getTotalMoney());
        BigDecimal round = NumberUtil.round(subMoney, 2);
        if (round.doubleValue() < 0) {
            payResponse.setCode(SbCode.MONEY_ERROR.getCode());
            payResponse.setMsg(SbCode.MONEY_ERROR.getMessage());
            return payResponse;
        }
        // 3。 够，就写入
        UserUpdateInfoRequest request = new UserUpdateInfoRequest();
        request.setId(userId);
        request.setMoney(round.doubleValue());
        tag = MqTags.PAY_MONEY_CANCLE.getTag();
        userService.updateUserInfo(request); // 暂时先不接受返回信息
        // 模拟异常
//            CastException.cast(SbCode.SYSTEM_ERROR);
        payResponse.setCode(SbCode.SUCCESS.getCode());
        payResponse.setMsg(SbCode.SUCCESS.getMessage());
        // 4. 按道理讲，这边更改订单状态......
        return payResponse;
    } catch (Exception e) {
        log.error("支付业务发生异常");
        MQDto mqDto = new MQDto();
        mqDto.setUserId(userId);
        mqDto.setUserMoney(userMoney);
        // 发送消息
        try {
            String key = RedisConstants.PAY_EXCEPTION_CANCLE_EXPIRE.getKey() + Convert.toStr(userId);
            sendCancelPay(topic,tag,key, JSON.toJSONString(mqDto));
            log.warn("支付回退消息已发送");
        } catch (Exception ex) {
            ex.printStackTrace();
            log.error("支付消息都崩的话...");
        }
        payResponse.setCode(SbCode.SYSTEM_ERROR.getCode());
        payResponse.setMsg(SbCode.SYSTEM_ERROR.getMessage());
        return payResponse;
    }

}
```

### payBack

```java
public PayResponse payBack(PayBackRequest request) {
    PayResponse response = new PayResponse();
    try {
        // 1. 退回金额
        // 读用户金额
        UserRequest userRequest = new UserRequest();
        userRequest.setId(request.getUserId());
        UserResponse userResponse = userService.getUserById(userRequest);
        UserDto userDto = userResponse.getUserDto();
        // 计算金额
        BigDecimal add = NumberUtil.add(userDto.getMoney() + request.getTotalMoney());
        BigDecimal round = NumberUtil.round(add, 2);
        // 写回
        UserUpdateInfoRequest userUpdateInfoRequest = new UserUpdateInfoRequest();
        userUpdateInfoRequest.setId(request.getUserId());
        userUpdateInfoRequest.setMoney(round.doubleValue());
        userService.updateUserInfo(userUpdateInfoRequest);
        // 2. 退回座位
        busService.filterRepeatSeats(request.getSeatsIds(), request.getCoundId());
        // 3. 更改订单状态：关闭
        OrderRequest orderRequest = new OrderRequest();
        orderRequest.setUuid(request.getOrderId());
        orderRequest.setOrderStatus("3");
        orderService.updateOrderStatus(orderRequest);
        response.setCode(SbCode.SUCCESS.getCode());
        response.setMsg(SbCode.SUCCESS.getMessage());
        return  response;
    } catch (Exception e) {
//            e.printStackTrace();
        log.error("退款业务异常");
        // 这里可以发消息， 此处先省略
        response.setCode(SbCode.SYSTEM_ERROR.getCode());
        response.setMsg(SbCode.SYSTEM_ERROR.getMessage());
        return response;
    }
}
```

这两个业务逻辑异常都采用了消息队列，日后再写。