# 用户服务

## 用户模块的MySQl数据库建表语句如下：

> 数据库名：school-bus （自定）
>
> DROP TABLE IF EXISTS `sb_user`;
> CREATE TABLE `sb_user` (
>  `uuid` int(11) NOT NULL AUTO_INCREMENT COMMENT '主键编号',
>  `user_name` varchar(50) NOT NULL DEFAULT '' COMMENT '用户账号',
>  `user_pwd` varchar(50) NOT NULL DEFAULT '' COMMENT '用户密码',
>  `nick_name` varchar(50) NOT NULL DEFAULT '' COMMENT '用户昵称',
>  `user_sex` int(11) DEFAULT NULL COMMENT '用户性别 0-男，1-女',
>  `email` varchar(50) NOT NULL DEFAULT '' COMMENT '用户邮箱',
>  `user_phone` varchar(50) NOT NULL DEFAULT '' COMMENT '用户手机号',
>  `begin_time` datetime NOT NULL COMMENT '创建时间',
>  `update_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT '修改时间',
>  `money` double NOT NULL COMMENT '用户余额',
>  `pay_password` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '支付密码',
>  PRIMARY KEY (`uuid`),
>  UNIQUE KEY `user_name` (`user_name`)
> ) ENGINE=InnoDB AUTO_INCREMENT=19 DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC COMMENT='用户表';
>
> INSERT INTO `sb_user` (`uuid`, `user_name`, `user_pwd`, `nick_name`, `user_sex`, `email`, `user_phone`, `begin_time`, `update_time`, `money`, `pay_password`) VALUES ('2', 'admin', '0192023a7bbd73250516f069df18b500', '隔壁泰山', '0', 'admin@qq.com', '13888888888', '2020-02-25 20:35:51', '2020-02-25 20:35:51', '0', ''),
> ('3', 'jiangzh', '5e2de6bd1c9b50f6e27d4e55da43b917', '阿里郎', '0', 'jiangzh@qq.com', '13866666666', '2020-02-25 20:35:51', '2020-02-25 20:35:51', '0', ''),

持久层框架使用`mybatis-plus`结合guns 4.2项目在`guns-rest`的test目录下提供的generator自动生成相关的SQL代码，很方便，具体使用请自行查看代码。

## UserController

`gateway`模块中`modular.user`下，与同目录下`form`和`auth`文件夹搭配使用，`form`提供网络请求的表单类，`auth`为guns自带的jwt验证，具体的验证逻辑可自行修改。

### checkUsername

> 根据提供的用户名去数据库中查找相关记录是否存在，找到就返回查找成功响应，没找到就返回错误响应。

### register

> 这个注意请求体的参数即可，也没什么可说的

### getUserById

> 注意缓存是否存在，存在直接返回即可，没有就调用业务`getUserById`获取结果

### updateUserInfo

> 注意请求体的参数即可，并且若是更新数据，那么自然要删除UserInfo的缓存，保持数据一致，写不写缓存这一点都行，下次请求getUserById会写缓存。

### logout
> 注意redis删除缓存即可


## UserServiceImpl

### checkUsername
```java
QueryWrapper<User> queryWrapper = new QueryWrapper<>();
// 查询user_name列与getUsername相匹配的数据
queryWrapper.eq("user_name", request.getUsername()); 
User user = userMapper.selectOne(queryWrapper);
```

### regsiter
```java
// 密码采用了md5加密
String md5Password = MD5Util.encrypt(user.getUserPwd());
user.setUserPwd(md5Password);
```

### login

```java
QueryWrapper<User> queryWrapper = new QueryWrapper<>();
// 根据user_name查询用户是否存在
queryWrapper.eq("user_name", request.getUsername());
User user = userMapper.selectOne(queryWrapper);
if (user != null && user.getUuid() > 0) {
    // 用户不为空，用户id > 0
    String md5Password = MD5Util.encrypt(request.getPassword());
    if (user.getUserPwd().equals(md5Password)) {
        // 数据库密码和所加密的md5密码一致
        res.setUserId(user.getUuid());
        res.setCode(SbCode.SUCCESS.getCode());
        res.setMsg(SbCode.SUCCESS.getMessage());
    }
}
```

### getUserById
```java
//根据id查询用户
User user = userMapper.selectById(request.getId());
UserDto userDto = userConverter.User2Res(user);
```

### updateUserInfo
```java
// 这里一采用了mapstruct映射，有兴趣的可以去网上找资料学习学习
User user = userConverter.res2User(request);
```

## 总结

用户服务，没有什么比较复杂的业务，复杂的业务在订单服务、场次服务、支付服务、
