
# 线程的创建方式
面试官：线程的创建方式有哪些？

我：我目前知道4种：分别如下：

- Thread


```java
public class Test extents Thread {
    public void run() {
      // 重写Thread的run方法
      System.out.println("dream");
    }
    
    public static void main(String[] args) {
        new Test().start();
    }
}
```

- Runnable

```java
public class Test {
    public static void main(String[] args) {
        new Thread(() -> {
            System.out.println("dream");
        }).start();
    } 
}
```

- Callable

```java
public class Test {
    public static void main(String[] args) {
        // FutureTask 构造方法包装了Callable和Runnable。
        FutureTask<Integer> task = new FutureTask<>(() -> {
            System.out.println("dream");
            return 0;
        });
        new Thread(task).start();
    }
}

```

- 线程池

```java

public class Test {
    public static void main(String[] args) {
        ExecutorService threadPool = Executors.newFixedThreadPool(1);
        threadPool.submit(() -> {
            System.out.println("dream");
        });
        threadPool.shutdown();
    }
}
```

面试官：Runnable和Callable有啥区别？

我：那得先看源码咯

请看

```java
@FunctionalInterface
public interface Runnable {
   /**
    * 被线程执行，没有返回值也无法抛出异常
    */
    public abstract void run();
}

@FunctionalInterface
public interface Callable<V> {
    /**
     * 计算结果，或在无法这样做时抛出异常。
     * @return 计算得出的结果
     * @throws 如果无法计算结果，则抛出异常
     */
    V call() throws Exception;
}
```

1. Runnable没有返回值并且无法抛出异常
2. 不巧，我Callable可以做到你不能做到的

线程池的源码后边讲，讲东西，要结合源码和例子讲！！！



> 补充一下主线程等待子线程的两种方式

# 主线程等待子线程的两种方式

## sleep
> 这个不常用，但是简单一些
```java
public class Test {
    void m() {
        System.out.println(Thread.currentThread().getName());
    }

    public static void main(String[] args) {
        Test t1 = new Test();
        for (int i = 0; i < 5; i++) {
            new Thread(t1::m, "Thread " + i).start();
        }
        try { TimeUnit.SECONDS.sleep(1); } catch (InterruptedException e) { e.printStackTrace(); }
        System.out.println("main thread");
    }
}
```

## join

```java
public class Test {
    void m() {
        System.out.println(Thread.currentThread().getName());
    }

    public static void main(String[] args) {
        Test t1 = new Test();
        ArrayList<Thread> threads = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            threads.add(new Thread(t1::m, "Thread " + i));
        }
        threads.forEach(o -> o.start());
        threads.forEach(o -> {
            try {
                o.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        });
        System.out.println("main thread");
    }
}
```

## CountDownLatch
```java
public class Test {
    private CountDownLatch latch;

    public Test(CountDownLatch latch) {
        this.latch = latch;
    }

    void m() {
        System.out.println(Thread.currentThread().getName());
        latch.countDown();
    }

    public static void main(String[] args) throws InterruptedException {
        CountDownLatch countDownLatch = new CountDownLatch(5);
        Test t1 = new Test(countDownLatch);
        for (int i = 0; i < 5; i++) {
            new Thread(t1::m, "Thread " + i).start();
        }
        countDownLatch.await();
        System.out.println("main thread");
    }
}
```
