## 单例模式
> 单例就不多说了

### 饿汉

```java
public class Singleton {
    private static Singleton instance = new Singleton();
    private Singleton(){}

    public static Singleton getInstance() {
        return instance;
    }
}
```
### 饿汉变种

```java
class Singleton {
    private static Singleton instance = null;
    static {
        instance = new Singleton();
    }
    private Singleton() {}

    public static Singleton getInstance() {
        return instance;
    }
}
```

### 懒汉(线程不安全)

```java
class Singleton {
    private static Singleton instance = null;
    private Singleton(){}

    public static Singleton getInstance() {
        if (instance == null){
            instance = new Singleton();
        }
        return instance;

    }
}
```

### 懒汉(线程安全，但消耗资源较为严重)

```java
class Singleton {
    private static Singleton instance = null;

    private Singleton() {
    }

    public static synchronized Singleton getInstance() {
        if (instance == null) {
            instance = new Singleton();
        }
        return instance;
    }
}
```

### 懒汉(线程安全，双重校验)

```java
class Singleton {
    private static volatile Singleton instance = null;

    private Singleton() {
    }

    public static Singleton getInstance() {
        if (instance == null) {
            synchronized (Singleton.class) {
                if (instance == null) {
                    instance = new Singleton();
                }
            }
        }
        return instance;
    }
}
```